#include "Curves.h"

#include <assert.h>

#include <math.h>

#ifndef WIN32
Curves *Curves::m_instance=NULL;
#endif

Curves::Curves(void)
{

};

Curves::Curves(std::istream & input_stream)
{
	fromStream(input_stream);
}

void Curves::toStream(std::ostream & output_stream)
{
	while (canZoomOut())
	{
		popZoom();		
	}
	
	output_stream << m_curves.size() << std::endl;
	
	for (CurveList::const_iterator curve_iterator = m_curves.begin(); curve_iterator != m_curves.end(); ++curve_iterator)
	{
		output_stream << *curve_iterator;
	}
}

void Curves::fromStream(std::istream & input_stream)
{
	unsigned int number_of_curves;

	input_stream >> number_of_curves;

	m_curves.resize(number_of_curves);

	for (unsigned int curve_index = 0; curve_index < number_of_curves; ++curve_index)
	{
		input_stream >> m_curves[curve_index];
	}
}

void Curves::addCurve(const std::string & curve_name, const double & minimum_value, const double & maximum_value, const double & start_value)
{
	Curve new_curve(curve_name, start_value);

	Zoomer * zoom = new_curve.getZoomer();

	zoom->setupObjectSpaceDimensions(zoom->getLeft(), zoom->getRight(), minimum_value, maximum_value);

	m_curves.push_back(new_curve);
}

Curve * Curves::getCurve(const std::string & curve_name)
{
	bool found_it = false;

	Curve * found_curve = NULL;

	for (CurveList::iterator curve_iterator = m_curves.begin(); (curve_iterator != m_curves.end()) && (! found_it); ++curve_iterator)
	{
		if (curve_iterator->getName() == curve_name)
		{
			found_curve = curve_iterator;

			found_it = true;
		}
	}
	
	assert(found_it);

	return found_curve;
}

Curves::const_iterator Curves::begin() const
{
	return m_curves.begin();
}

Curves::const_iterator Curves::end() const
{
	return m_curves.end();
}

Curves::iterator Curves::begin()
{
	return m_curves.begin();
}

Curves::iterator Curves::end()
{
	return m_curves.end();
}

unsigned int Curves::getNumberOfCurves(void) const
{
	return m_curves.size();
}

void Curves::clearCurves(void)
{
	m_curves.clear();
}

void Curves::setWrapControlPointsFlags(const bool & wrapping)
{
	for (CurveList::iterator curve_iterator = m_curves.begin(); curve_iterator != m_curves.end(); ++curve_iterator)
	{
		curve_iterator->setWrapControlPointsFlag(wrapping);
	}
}

void Curves::getClosestControlPointOnShownCurve(const Point & point, double * closest_control_point_distance, Curve * * closest_control_point_curve, unsigned int * closest_control_point_index)
{
	static const double MAXIMUM_DISTANCE = 9e99;

	*closest_control_point_distance = MAXIMUM_DISTANCE;

	for (CurveList::iterator curve_iterator = m_curves.begin(); curve_iterator != m_curves.end(); ++curve_iterator)
	{
		if (curve_iterator->isShown())
		{
			double distance;

			unsigned int closest_point_index_on_this_curve = curve_iterator->getClosestControlPoint(point, &distance);

			if (distance < *closest_control_point_distance)
			{
				*closest_control_point_index		= closest_point_index_on_this_curve;
				*closest_control_point_distance	= distance;
				*closest_control_point_curve		= curve_iterator;
			}
		}
	}

	closest_control_point_index = closest_control_point_index;
}

void Curves::getClosestShownCurve(const Point & point, double * closest_curve_distance, Curve * * closest_curve)
{
	static const double MAXIMUM_DISTANCE = 9e99;

	*closest_curve_distance = MAXIMUM_DISTANCE;

	for (CurveList::iterator curve_iterator = m_curves.begin(); curve_iterator != m_curves.end(); ++curve_iterator)
	{
		if (curve_iterator->isShown())
		{
			double distance = curve_iterator->getDistanceToCurve(point);

			if (distance < *closest_curve_distance)
			{
				*closest_curve_distance		= distance;
				*closest_curve				= curve_iterator;
			}
		}
	}
}

void Curves::startZoomSelection(const double & left, const double & bottom)
{
	for (CurveList::iterator curve_iterator = m_curves.begin(); curve_iterator != m_curves.end(); ++curve_iterator)
	{	
		Zoomer * zoomer = curve_iterator->getZoomer();
		
		zoomer->startZoomSelection(left, bottom);
	}
}

void Curves::finishZoomSelection(const double & right, const double & top)
{
	for (CurveList::iterator curve_iterator = m_curves.begin(); curve_iterator != m_curves.end(); ++curve_iterator)
	{	
		Zoomer * zoomer = curve_iterator->getZoomer();
		
		zoomer->finishZoomSelection(right, top);
	}
}

void Curves::pushZoom(void)
{
	for (CurveList::iterator curve_iterator = m_curves.begin(); curve_iterator != m_curves.end(); ++curve_iterator)
	{	
		Zoomer * zoomer = curve_iterator->getZoomer();
		
		zoomer->pushZoom();
	}
}

void Curves::popZoom(void)
{
	for (CurveList::iterator curve_iterator = m_curves.begin(); curve_iterator != m_curves.end(); ++curve_iterator)
	{	
		curve_iterator->markDirty();

		Zoomer * zoomer = curve_iterator->getZoomer();
		
		zoomer->popZoom();
	}
}

bool Curves::canZoomOut(void) const
{
	// make sure all can zoom out;

	bool can_zoom_out = true;

	if (m_curves.empty())
	{
		can_zoom_out = false;
	}
	else
	{
		for (CurveList::const_iterator curve_iterator = m_curves.begin(); curve_iterator != m_curves.end(); ++curve_iterator)
		{	
			const Zoomer * zoomer = curve_iterator->getZoomer();
			
			if (! zoomer->canZoomOut())
			{
				can_zoom_out = false;	
				
			}
		}
	}

	return can_zoom_out;
}

void Curves::changeAnimationLength(const double & animation_length)
{
	for (CurveList::iterator curve_iterator = m_curves.begin(); curve_iterator != m_curves.end(); ++curve_iterator)
	{	
		curve_iterator->markDirty();

		Zoomer * zoomer = curve_iterator->getZoomer();

		zoomer->setupObjectSpaceDimensions(zoomer->getLeft(), animation_length, zoomer->getBottom(), zoomer->getTop());
	}
}

double Curves::getTime(const double & normalized_x) const
{
	assert(normalized_x >= 0.0);
	assert(normalized_x <= 1.0);
	
	double time = 0.0;

	if (! m_curves.empty())
	{
		const Zoomer * zoomer = m_curves.begin()->getZoomer();

		double m_start_time = zoomer->getLeft();
		double m_end_time = zoomer->getRight();

		double m_time_range = m_end_time - m_start_time;

		time = (1 - normalized_x) * m_start_time + (normalized_x) * m_end_time;
	}

	return time;
}

void Curves::drawAllCurves(void) const
{
	for (CurveList::const_iterator curve_iterator = m_curves.begin(); curve_iterator != m_curves.end(); ++curve_iterator)
	{
		curve_iterator->drawCurve();
	}
}

std::ostream & operator<<(std::ostream & output_stream, Curves & curves)
{
	curves.toStream(output_stream);

	return output_stream;
}

std::istream & operator>>(std::istream & input_stream, Curves & curves)
{
	curves.fromStream(input_stream);

	return input_stream;
}
