// 
// scene.cpp
// 

#include <stdio.h>

#include "scene.h"
#include "aniDoc.h"
#include "aniUI.h"
#include "modeler.h"
#include "ctrlPanel.h"

#include "fltkCurveEditor.h"

extern CAniUI *aniUI;

Scene::Scene(Object* obj) : objects() 
{
	m_nObjCnt=0;
	m_pView=NULL;
	m_pObjList=NULL;
	m_pActiveObj=NULL;
	add( obj );
}

Scene::~Scene()
{
	if (m_pObjList) {
		m_pObjList->hide();
		delete m_pObjList;
	}
	if (m_pView) {
		m_pView->hide();
		aniUI->removeView(m_pView);
		delete m_pView;
	}

	for( giter g = objects.begin(); g != objects.end(); ++g ) {
		delete (*g);
	}
}

void Scene::createView()
{
	if (m_pView!=NULL) delete m_pView;

	// create a view for it 
	m_pView = new CGLView(600, 100, 400, 400, "Scene", this);
	m_pView->setUI(m_pDoc->getUI());
	m_pView->resizable(m_pView);
}

void Scene::createObjList()
{
	if (m_pObjList!=NULL) delete m_pObjList;

	// create the object list panel for the scene
	int i=0;
	giter g;

	m_pObjList=new Fl_Window(100, 100, 200, 20*(m_nObjCnt+1)+10, "Object List");

		Fl_Box *box=new Fl_Box(5, 5, 190, 18, "             Object                V   A");

		for( g = objects.begin(); g != objects.end(); ++g ) {
			i++;
			// create buttons for object
			char *buf=new char[16];

			sprintf(buf, "%s<%d>", (*g)->getObjName(), (*g)->getObjID());
			Fl_Button *b1=new Fl_Button(10, i*20+5, 150, 18, buf);
			b1->user_data(new scene_params((void*)this,(Object*)(*g)));
			b1->callback(cb_fork_object);

			Fl_Check_Button *b2=new Fl_Check_Button(160, i*20+5, 18, 18, "");
			b2->value(1);
			b2->user_data(new scene_params((void*)this,(Object*)(*g)));
			b2->callback(cb_toggle_object);
		}

		i=0;
		Fl_Group* o=new Fl_Group(180, 20, 20, 20*(m_nObjCnt));
		for( g = objects.begin(); g != objects.end(); ++g ) {
			i++;
			Fl_Check_Button *b3=new Fl_Check_Button(180, i*20+5, 18, 18, "");
			b3->type(102);
			b3->user_data(new scene_params((void*)this,(Object*)(*g)));
			b3->callback(cb_activate_object);
			if (i==1) b3->value(1);
		}
		o->end();

	m_pObjList->end();

	m_pActiveObj=objects.front();
	aniUI->m_pCurveEditor->resetCurveMenu(m_pActiveObj);
}

void Scene::saveImage(char *iname) 
{ 
	m_pView->saveImage(iname); 
}

void Scene::show()
{
	if (m_pView) m_pView->show();

/* we only support one object now */
//	if (m_pObjList) m_pObjList->show();
	// if the object doesn't currently have a subwindow, show the panel and create a scene for the object
	if (!(m_pActiveObj->m_pPanel->visible())) {
		m_pActiveObj->m_pPanel->show();
		aniUI->addPanel(m_pActiveObj->m_pPanel);
	}

	// open control depending on the mode
	if (g_mode==CURVE_MODE) {

		// show fltkCurveEditor
		aniUI->m_pCurveEditor->show();
		aniUI->m_pCurveEditor->setScene(aniUI->getDocument()->getScene());
	
		// hide the object panel
		if (m_pDoc->getScene()) {
			m_pDoc->getScene()->m_pActiveObj->m_pPanel->hide();
		}

		aniUI->refreshAll();
	} else {
		// hide fltkCurveEditor
		aniUI->m_pCurveEditor->hide();

		// if the object doesn't currently have a subwindow, show the panel and create a scene for the object
		if (m_pDoc->getScene()) {
			m_pDoc->getScene()->m_pActiveObj->m_pPanel->show();
			aniUI->addPanel(m_pDoc->getScene()->m_pActiveObj->m_pPanel);
		}

		aniUI->refreshAll();
	}

	aniUI->addView(m_pView);
}

void Scene::redraw(int drawMode)
{
	draw_mode_setup(drawMode);

	for( giter g = objects.begin(); g != objects.end(); ++g ) {
		if ((*g)->isVisible())
			(*g)->redraw((*g)->getObjID(),drawMode);
	}

	draw_mode_finish(drawMode);
}

void Scene::refresh()
{
	m_pView->refresh();
}

/* draw_mode_setup
 *
 * models should call this after clearing the window in their redraw
 * function.  it sets up default material and light properties, and
 * does initialization specific to the drawing mode.
 */

void Scene::draw_mode_setup(int drawMode)
{
	setDrawMode( drawMode );

    GLfloat light_position[] = { 10.0f, 10.0f, 10.0f, 0.0f };

	GLfloat mat_ambient[4];
	GLfloat mat_diffuse[4];
	GLfloat mat_specular[4];
	GLfloat mat_shininess;

    mat_ambient[0] = mat_ambient[1] = mat_ambient[2] = 0.5;
    mat_ambient[3] = 1.0;
    mat_diffuse[0] = mat_diffuse[1] = mat_diffuse[2] = 0.5;
    mat_diffuse[3] = 1.0;
    mat_specular[0] = mat_specular[1] = mat_specular[2] = mat_specular[3] = 1.0;
    mat_shininess = 100.0;

    glMatrixMode( GL_MODELVIEW );
    glLoadIdentity();
    
    glEnable( GL_DEPTH_TEST );

    if ( drawMode & MODE_NORMAL )
    {
		/* initialization for "normal" mode. */
	
		glShadeModel( GL_SMOOTH );
		glMaterialfv( GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, mat_ambient );
		glMaterialfv( GL_FRONT_AND_BACK, GL_SPECULAR, mat_specular );
		glMaterialfv( GL_FRONT_AND_BACK, GL_SHININESS, &mat_shininess );
		glLightfv( GL_LIGHT0, GL_POSITION, light_position );

		glEnable( GL_LIGHTING );
		glEnable( GL_LIGHT0 );
		glEnable( GL_NORMALIZE );
	
		glLightModeli( GL_LIGHT_MODEL_TWO_SIDE, GL_FALSE );
		glPolygonMode( GL_FRONT, GL_FILL );
		glCullFace( GL_BACK );
		glEnable( GL_CULL_FACE ); 
    }

    if ( drawMode & MODE_WIREFRAME )
    {
		/* initialization for "wireframe" mode. */
	
		glDisable( GL_LIGHTING );
		glDisable( GL_NORMALIZE );
	
		glPolygonMode( GL_FRONT_AND_BACK, GL_LINE );

		glColor3f( 0.5, 0.5, 0.5 );
    }

    if ( drawMode & MODE_FLATSHADE )
    {
		/* initialization for "flat shaded" mode. */
	
		glDisable( GL_LIGHTING );
		glDisable( GL_NORMALIZE );
	
		glPolygonMode( GL_FRONT_AND_BACK, GL_FILL );
	
		glColor3f( 0.5, 0.5, 0.5 );
    }
/*    
    if ( drawMode & MODE_RAYTRACE )
    {
		FILE* f;
		Tcl_Obj* fn;

		fn = Tcl_ObjGetVar2( g_interp, g_varRayfile, NULL, TCL_GLOBAL_ONLY );

		Debug( "output file is [%s]\n", Tcl_GetStringFromObj( fn, NULL ) );
		
		f = fopen( Tcl_GetStringFromObj( fn, NULL ), "w" );
		if ( f == NULL )
		{
			Tcl_SetResult( g_interp, "can't open raytrace output file", TCL_STATIC );
			return TCL_ERROR;
		}

		fprintf( f, "SBT-raytracer 1.0\n\n" );
		fprintf( f, "camera { fov=30; }\n\n" );
		fprintf( f, "directional_light { direction=(-1,-1,-1); color=(0.7,0.7,0.7); }\n\n" );
		 

		g_rayfile = f;
    }
*/	      

}

void Scene::draw_mode_finish( int drawMode )
{
/*
    if ( drawMode & MODE_RAYTRACE )
    {
		fclose( g_rayfile );
		g_rayfile = NULL;
    }
*/
}

void Scene::cb_fork_object(Fl_Widget* o, void* v)
{
	scene_params* args=(scene_params *)(o->user_data());

	Scene  *scene=(Scene*)(args->self);
	Object *obj=(Object*)(args->obj);

	// if the object doesn't currently have a subwindow, show the panel and create a scene for the object
	if (!(obj->m_pPanel->visible())) {
		obj->m_pPanel->show();
		aniUI->addPanel(obj->m_pPanel);
	}
}

void Scene::cb_toggle_object(Fl_Widget* o, void* v)
{
	scene_params* args=(scene_params *)(o->user_data());

	Scene  *scene=(Scene*)(args->self);
	Object *obj=(Object*)(args->obj);

	obj->toggleVisibility();
	scene->refresh();
}

void Scene::cb_activate_object(Fl_Widget* o, void* v)
{
	scene_params* args=(scene_params *)(o->user_data());

	Scene  *scene=(Scene*)(args->self);
	Object *obj=(Object*)(args->obj);

	scene->m_pActiveObj=obj;

	aniUI->m_pCurveEditor->resetCurveMenu(scene->m_pActiveObj);
}
